--
-- VHDL course - Practical Phase III - Autumn 2002
--
-- Group 3:
--  Jouni Osmala <josmala@cc.hut.fi>
--  Ilpo Jrvinen <ilpo.jarvinen@helsinki.fi>
--

library ieee;
use ieee.std_logic_1164.all;
use work.wnm_types.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity DSP is
port (
	clk: in std_logic;
	reset: in std_logic;
	wnm_block_select: out std_logic;
	wnm_write_enable: out std_logic;
	wnm_address:out std_logic_vector(7 downto 0);
	wnm_data_out:out std_logic_vector(11 downto 0);
	wnm_data_in: in std_logic_vector(11 downto 0);
	wnm_alarm: in std_logic;

	command: in instruction);
end entity DSP;

architecture behave of DSP is

	signal prev_command: instruction;
	signal registers: regbank;
begin
	
	process (clk,reset) 
	begin
		if reset='1' then
		wnm_block_select<='0';
		wnm_address<="00000000";
		wnm_data_out<="000000000000";
		wnm_write_enable<='0';
		
		elsif(rising_edge(clk)) then 
		if prev_command.opcode = wnm_read then 
			registers(conv_integer(prev_command.reg3))<=conv_integer(signed(wnm_data_in));	
		elsif prev_command.opcode = wnm_write then 
			wnm_write_enable <='0';
		end if;
		if command.opcode = wnm_read then  --read wnm 
			wnm_address<=command.reg1;
			wnm_write_enable<='0';
			wnm_block_select<='1';
		end if;
		
		if command.opcode= wnm_write then --write wnm
		wnm_address<=command.reg1;
		wnm_write_enable<='1';
		wnm_block_select<='1';
		wnm_data_out<=conv_std_logic_vector(registers(conv_integer(command.reg2)),12);
		else wnm_block_select<='0';
		end if;
		if command.opcode= wnm_write_im then
		wnm_address<=command.reg1;
		wnm_write_enable<='1';
		wnm_block_select<='1';
		wnm_data_out(7 downto 0)<=command.reg2;
		wnm_data_out(11 downto 8) <= b"0000";
		else wnm_block_select<='0';
		end if;
		
		if command.opcode=dsp_add then 
			registers(conv_integer(command.reg3))<=registers(conv_integer(command.reg2))+registers(conv_integer(command.reg1));
		end if;
		if command.opcode=dsp_sub then 
			registers(conv_integer(command.reg3))<=registers(conv_integer(command.reg2))-registers(conv_integer(command.reg1));
		end if;
		if command.opcode=dsp_mul then 
			registers(conv_integer(command.reg3))<=registers(conv_integer(command.reg2))*registers(conv_integer(command.reg1));
		end if;
		if command.opcode=dsp_div then 
			registers(conv_integer(command.reg3))<=registers(conv_integer(command.reg2))/registers(conv_integer(command.reg1));
		end if;
		prev_command<=command;
		end if;
	end process;
end behave;