--
-- VHDL course - Practical Phase III - Autumn 2002
--
-- Group 3: 
--  Jouni Osmala <josmala@cc.hut.fi>
--  Ilpo Jrvinen <ilpo.jarvinen@helsinki.fi>
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity decoder is
  generic (
    inputs : integer;                                -- Width of the input
    outputs : integer);                              -- Width of the output

  port (
    din  : in std_logic_vector(inputs-1 downto 0);   -- Data input
    dout : out std_logic_vector(outputs-1 downto 0); -- Data output
    err  : out std_logic);                           -- din overflow.

end decoder;

ARCHITECTURE behave OF decoder IS
	
begin

  -- purpose: din is decoded to dout. If outputs is less than 2^inputs the 
  -- err is used as output of all missing output wires.
  --
  -- inputs: din
  -- outputs: dout, err
  process (din) is
    variable idx : integer;
  begin
    assert outputs <= (2 ** inputs) 
      report "Unnecessary output wires in " & 
        integer'image(inputs) &
        " to " &
        integer'image(outputs) &
        "decoder, where 2^" &
        integer'image(inputs) &
        " = " &
        integer'image(2 ** inputs)
      severity note;
    idx := conv_integer(unsigned(din));
    
    for i in outputs-1 downto 0 loop
      if idx = i then
        dout(i) <= '1';
      else
        dout(i) <= '0';
      end if;
    end loop;
    
    if idx > outputs-1 then
      err <= '1';
    else
      err <= '0';
    end if;
  end process;
end behave;
