--
-- VHDL course - Practical Phase III - Autumn 2002
--
-- Group 3: 
--  Jouni Osmala <josmala@cc.hut.fi>
--  Ilpo Jrvinen <ilpo.jarvinen@helsinki.fi>
--

library ieee;
use ieee.std_logic_1164.all;

entity regn is
  
  generic (
    bits : integer := 8);                              -- Width of the reg

  port (
    clk   : in  std_logic;
    reset : in  std_logic;                             -- Asynchronious reset
    
    load  : in  std_logic;                             -- Load din into reg
    clear : in  std_logic;                             -- Synchronious clear

    din   : in     std_logic_vector(bits-1 downto 0);  -- Data input
    dout  : buffer std_logic_vector(bits-1 downto 0)); -- Data output & store


end regn;

architecture behave of regn is

begin  -- behave

  -- purpose: When reset is '1' the register is asynchroniously cleared.
  --          If the reset is '0' the register will function synchroniously:
  --          clear and load signals are used to control synchronious 
  --          operation. The clear has higher precedence than load.
  --
  -- inputs: din, reset, clk, load, clear
  -- outputs: dout
  process (reset, clk)
  begin  -- process
    if reset = '1' then
	dout <= (others => '0');
    elsif rising_edge(clk) then
      if clear = '1' then
        dout <= (others => '0');
      elsif load = '1' then
        dout <= din;
      else
        -- Always load the register on the clock edge.
        dout <= dout;
      end if;
    end if;
  end process;

end behave;
