--
-- VHDL course - Practical Phase III - Autumn 2002
--
-- Group 3: 
--  Jouni Osmala <josmala@cc.hut.fi>
--  Ilpo Jrvinen <ilpo.jarvinen@helsinki.fi>
--

library ieee;
use ieee.std_logic_1164.all;

package wnm_types is
  constant reg_size : integer := 19;
  constant result_size : integer := reg_size-7;
  constant reg_count : integer := 12;
  
  subtype  reg_index is integer range reg_count-1 downto 0;

  -- Integrator registers...
  subtype  reg_t is std_logic_vector(reg_size-1 downto 0);
  type     reg_array is array (reg_index) of reg_t;
  constant reg_zero : reg_t := (others => '0');

  -- Divided by 128...
  subtype  result_t is std_logic_vector(result_size-1 downto 0);
  type     result_array is array (reg_index) of result_t;
  constant result_zero : result_t := (others => '0');

   --DSP types--  
  subtype dsp_opcode is std_logic_vector(7 downto 0);
  type instruction is record
	opcode: dsp_opcode;
	reg1:   std_logic_vector(7 downto 0);
	reg2:   std_logic_vector(7 downto 0);
	reg3:   std_logic_vector(7 downto 0);
  end record instruction;

  type regbank is array(0 to 256) of integer;
 
  constant dsp_nop: 			dsp_opcode:= "00000000";   
  constant wnm_write: 			dsp_opcode:= "00000001"; 
  constant wnm_write_im:                dsp_opcode:= "00001010";
  constant wnm_read: 			dsp_opcode:= "00000010";
  constant wnm_select:			dsp_opcode:= "00000011";
  constant wnm_deselect:                dsp_opcode:= "00000100";
  constant dsp_add: 		  	dsp_opcode:= "00000101";
  constant dsp_sub: 		  	dsp_opcode:= "00000110";
  constant dsp_mul: 		  	dsp_opcode:= "00000111";
  constant dsp_div: 		  	dsp_opcode:= "00001000";
  constant dsp_mac:		        dsp_opcode:= "00001001";
end wnm_types;


